@@ -10,7 +10,7 @@
 
 /*
  *
- * Copyright (c) 2013, Cisco Systems, Inc.
+ * Copyright (c) 2013-2017, Cisco Systems, Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,35 +45,27 @@
  */
 
 #ifdef HAVE_CONFIG_H
-    #include <config.h>
+#include <config.h>
 #endif
 
 #include <openssl/evp.h>
-#include "aes_icm_ossl.h"
-#include "aes_gcm_ossl.h"
+#include "aes_gcm.h"
 #include "alloc.h"
+#include "err.h" /* for srtp_debug */
 #include "crypto_types.h"
-
+#include "cipher_types.h"
 
 srtp_debug_module_t srtp_mod_aes_gcm = {
-    0,               /* debugging is off by default */
-    "aes gcm"        /* printable module name       */
+    0,        /* debugging is off by default */
+    "aes gcm" /* printable module name       */
 };
 
-/*
- * The following are the global singleton instances for the
- * 128-bit and 256-bit GCM ciphers.
- */
-extern const srtp_cipher_type_t srtp_aes_gcm_128_openssl;
-extern const srtp_cipher_type_t srtp_aes_gcm_256_openssl;
-
 /*
  * For now we only support 8 and 16 octet tags.  The spec allows for
  * optional 12 byte tag, which may be supported in the future.
  */
-#define GCM_AUTH_TAG_LEN    16
-#define GCM_AUTH_TAG_LEN_8  8
-
+#define GCM_AUTH_TAG_LEN 16
+#define GCM_AUTH_TAG_LEN_8 8
 
 /*
  * This function allocates a new instance of this crypto engine.
@@ -82,23 +74,25 @@ extern const srtp_cipher_type_t srtp_aes_gcm_256_openssl;
  * key length includes the 14 byte salt value that is used when
  * initializing the KDF.
  */
-static srtp_err_status_t srtp_aes_gcm_openssl_alloc (srtp_cipher_t **c, int key_len, int tlen)
+static srtp_err_status_t srtp_aes_gcm_openssl_alloc(srtp_cipher_t **c,
+                                                    int key_len,
+                                                    int tlen)
 {
     srtp_aes_gcm_ctx_t *gcm;
 
-    debug_print(srtp_mod_aes_gcm, "allocating cipher with key length %d", key_len);
+    debug_print(srtp_mod_aes_gcm, "allocating cipher with key length %d",
+                key_len);
     debug_print(srtp_mod_aes_gcm, "allocating cipher with tag length %d", tlen);
 
     /*
      * Verify the key_len is valid for one of: AES-128/256
      */
-    if (key_len != SRTP_AES_128_GCM_KEYSIZE_WSALT &&
-        key_len != SRTP_AES_256_GCM_KEYSIZE_WSALT) {
+    if (key_len != SRTP_AES_GCM_128_KEY_LEN_WSALT &&
+        key_len != SRTP_AES_GCM_256_KEY_LEN_WSALT) {
         return (srtp_err_status_bad_param);
     }
 
-    if (tlen != GCM_AUTH_TAG_LEN &&
-        tlen != GCM_AUTH_TAG_LEN_8) {
+    if (tlen != GCM_AUTH_TAG_LEN && tlen != GCM_AUTH_TAG_LEN_8) {
         return (srtp_err_status_bad_param);
     }
 
@@ -107,56 +101,60 @@ static srtp_err_status_t srtp_aes_gcm_openssl_alloc (srtp_cipher_t **c, int key_
     if (*c == NULL) {
         return (srtp_err_status_alloc_fail);
     }
-    memset(*c, 0x0, sizeof(srtp_cipher_t));
 
     gcm = (srtp_aes_gcm_ctx_t *)srtp_crypto_alloc(sizeof(srtp_aes_gcm_ctx_t));
     if (gcm == NULL) {
-	srtp_crypto_free(*c);	
-	*c = NULL;
+        srtp_crypto_free(*c);
+        *c = NULL;
         return (srtp_err_status_alloc_fail);
     }
-    memset(gcm, 0x0, sizeof(srtp_aes_gcm_ctx_t));
+
+    gcm->ctx = EVP_CIPHER_CTX_new();
+    if (gcm->ctx == NULL) {
+        srtp_crypto_free(gcm);
+        srtp_crypto_free(*c);
+        *c = NULL;
+        return srtp_err_status_alloc_fail;
+    }
 
     /* set pointers */
     (*c)->state = gcm;
 
     /* setup cipher attributes */
     switch (key_len) {
-    case SRTP_AES_128_GCM_KEYSIZE_WSALT:
-        (*c)->type = &srtp_aes_gcm_128_openssl;
-        (*c)->algorithm = SRTP_AES_128_GCM;
-        gcm->key_size = SRTP_AES_128_KEYSIZE;
+    case SRTP_AES_GCM_128_KEY_LEN_WSALT:
+        (*c)->type = &srtp_aes_gcm_128;
+        (*c)->algorithm = SRTP_AES_GCM_128;
+        gcm->key_size = SRTP_AES_128_KEY_LEN;
         gcm->tag_len = tlen;
         break;
-    case SRTP_AES_256_GCM_KEYSIZE_WSALT:
-        (*c)->type = &srtp_aes_gcm_256_openssl;
-        (*c)->algorithm = SRTP_AES_256_GCM;
-        gcm->key_size = SRTP_AES_256_KEYSIZE;
+    case SRTP_AES_GCM_256_KEY_LEN_WSALT:
+        (*c)->type = &srtp_aes_gcm_256;
+        (*c)->algorithm = SRTP_AES_GCM_256;
+        gcm->key_size = SRTP_AES_256_KEY_LEN;
         gcm->tag_len = tlen;
         break;
     }
 
     /* set key size        */
     (*c)->key_len = key_len;
-    EVP_CIPHER_CTX_init(&gcm->ctx);
 
     return (srtp_err_status_ok);
 }
 
-
 /*
  * This function deallocates a GCM session
  */
-static srtp_err_status_t srtp_aes_gcm_openssl_dealloc (srtp_cipher_t *c)
+static srtp_err_status_t srtp_aes_gcm_openssl_dealloc(srtp_cipher_t *c)
 {
     srtp_aes_gcm_ctx_t *ctx;
 
-    ctx = (srtp_aes_gcm_ctx_t*)c->state;
+    ctx = (srtp_aes_gcm_ctx_t *)c->state;
     if (ctx) {
-        EVP_CIPHER_CTX_cleanup(&ctx->ctx);
-	/* zeroize the key material */
-	octet_string_set_to_zero((uint8_t*)ctx, sizeof(srtp_aes_gcm_ctx_t));
-	srtp_crypto_free(ctx);
+        EVP_CIPHER_CTX_free(ctx->ctx);
+        /* zeroize the key material */
+        octet_string_set_to_zero(ctx, sizeof(srtp_aes_gcm_ctx_t));
+        srtp_crypto_free(ctx);
     }
 
     /* free memory */
@@ -171,48 +169,22 @@ static srtp_err_status_t srtp_aes_gcm_openssl_dealloc (srtp_cipher_t *c)
  *
  * the key is the secret key
  */
-static srtp_err_status_t srtp_aes_gcm_openssl_context_init (srtp_aes_gcm_ctx_t *c, const uint8_t *key)
-{
-    c->dir = direction_any;
-
-    /* copy key to be used later when CiscoSSL crypto context is created */
-    v128_copy_octet_string((v128_t*)&c->key, key);
-
-    if (c->key_size == SRTP_AES_256_KEYSIZE) {
-        debug_print(srtp_mod_aes_gcm, "Copying last 16 bytes of key: %s",
-                    v128_hex_string((v128_t*)(key + SRTP_AES_128_KEYSIZE)));
-        v128_copy_octet_string(((v128_t*)(&c->key.v8)) + 1,
-                               key + SRTP_AES_128_KEYSIZE);
-    }
-
-    debug_print(srtp_mod_aes_gcm, "key:  %s", v128_hex_string((v128_t*)&c->key));
-
-    EVP_CIPHER_CTX_cleanup(&c->ctx);
-
-    return (srtp_err_status_ok);
-}
-
-
-/*
- * aes_gcm_openssl_set_iv(c, iv) sets the counter value to the exor of iv with
- * the offset
- */
-static srtp_err_status_t srtp_aes_gcm_openssl_set_iv (srtp_aes_gcm_ctx_t *c, uint8_t *iv, int direction)
+static srtp_err_status_t srtp_aes_gcm_openssl_context_init(void *cv,
+                                                           const uint8_t *key)
 {
+    srtp_aes_gcm_ctx_t *c = (srtp_aes_gcm_ctx_t *)cv;
     const EVP_CIPHER *evp;
 
-    if (direction != direction_encrypt && direction != direction_decrypt) {
-        return (srtp_err_status_bad_param);
-    }
-    c->dir = direction;
+    c->dir = srtp_direction_any;
 
-    debug_print(srtp_mod_aes_gcm, "setting iv: %s", v128_hex_string((v128_t*)iv));
+    debug_print(srtp_mod_aes_gcm, "key:  %s",
+                srtp_octet_string_hex_string(key, c->key_size));
 
     switch (c->key_size) {
-    case SRTP_AES_256_KEYSIZE:
+    case SRTP_AES_256_KEY_LEN:
         evp = EVP_aes_256_gcm();
         break;
-    case SRTP_AES_128_KEYSIZE:
+    case SRTP_AES_128_KEY_LEN:
         evp = EVP_aes_128_gcm();
         break;
     default:
@@ -220,19 +192,40 @@ static srtp_err_status_t srtp_aes_gcm_openssl_set_iv (srtp_aes_gcm_ctx_t *c, uin
         break;
     }
 
-    if (!EVP_CipherInit_ex(&c->ctx, evp, NULL, (const unsigned char*)&c->key.v8,
-                           NULL, (c->dir == direction_encrypt ? 1 : 0))) {
+    EVP_CIPHER_CTX_cleanup(c->ctx);
+    if (!EVP_CipherInit_ex(c->ctx, evp, NULL, key, NULL, 0)) {
         return (srtp_err_status_init_fail);
     }
 
-    /* set IV len  and the IV value, the followiong 3 calls are required */
-    if (!EVP_CIPHER_CTX_ctrl(&c->ctx, EVP_CTRL_GCM_SET_IVLEN, 12, 0)) {
+    return (srtp_err_status_ok);
+}
+
+/*
+ * aes_gcm_openssl_set_iv(c, iv) sets the counter value to the exor of iv with
+ * the offset
+ */
+static srtp_err_status_t srtp_aes_gcm_openssl_set_iv(
+    void *cv,
+    uint8_t *iv,
+    srtp_cipher_direction_t direction)
+{
+    srtp_aes_gcm_ctx_t *c = (srtp_aes_gcm_ctx_t *)cv;
+
+    if (direction != srtp_direction_encrypt &&
+        direction != srtp_direction_decrypt) {
+        return (srtp_err_status_bad_param);
+    }
+    c->dir = direction;
+
+    debug_print(srtp_mod_aes_gcm, "setting iv: %s",
+                srtp_octet_string_hex_string(iv, 12));
+
+    if (!EVP_CIPHER_CTX_ctrl(c->ctx, EVP_CTRL_GCM_SET_IVLEN, 12, 0)) {
         return (srtp_err_status_init_fail);
     }
-    if (!EVP_CIPHER_CTX_ctrl(&c->ctx, EVP_CTRL_GCM_SET_IV_FIXED, -1, (void*)iv)) {
-        return (srtp_err_status_init_fail);
-    }
-    if (!EVP_CIPHER_CTX_ctrl(&c->ctx, EVP_CTRL_GCM_IV_GEN, 0, (void*)iv)) {
+
+    if (!EVP_CipherInit_ex(c->ctx, NULL, NULL, NULL, iv,
+                           (c->dir == srtp_direction_encrypt ? 1 : 0))) {
         return (srtp_err_status_init_fail);
     }
 
@@ -247,10 +240,16 @@ static srtp_err_status_t srtp_aes_gcm_openssl_set_iv (srtp_aes_gcm_ctx_t *c, uin
  *	aad	Additional data to process for AEAD cipher suites
  *	aad_len	length of aad buffer
  */
-static srtp_err_status_t srtp_aes_gcm_openssl_set_aad (srtp_aes_gcm_ctx_t *c, const uint8_t *aad, uint32_t aad_len)
+static srtp_err_status_t srtp_aes_gcm_openssl_set_aad(void *cv,
+                                                      const uint8_t *aad,
+                                                      uint32_t aad_len)
 {
+    srtp_aes_gcm_ctx_t *c = (srtp_aes_gcm_ctx_t *)cv;
     int rv;
 
+    debug_print(srtp_mod_aes_gcm, "setting AAD: %s",
+                srtp_octet_string_hex_string(aad, aad_len));
+
     /*
      * Set dummy tag, OpenSSL requires the Tag to be set before
      * processing AAD
@@ -258,15 +257,15 @@ static srtp_err_status_t srtp_aes_gcm_openssl_set_aad (srtp_aes_gcm_ctx_t *c, co
 
     /*
      * OpenSSL never write to address pointed by the last parameter of
-     * EVP_CIPHER_CTX_ctrl while EVP_CTRL_GCM_SET_TAG (in reality, 
-     * OpenSSL copy its content to the context), so we can make 
+     * EVP_CIPHER_CTX_ctrl while EVP_CTRL_GCM_SET_TAG (in reality,
+     * OpenSSL copy its content to the context), so we can make
      * aad read-only in this function and all its wrappers.
      */
     unsigned char dummy_tag[GCM_AUTH_TAG_LEN];
     memset(dummy_tag, 0x0, GCM_AUTH_TAG_LEN);
-    EVP_CIPHER_CTX_ctrl(&c->ctx, EVP_CTRL_GCM_SET_TAG, c->tag_len, &dummy_tag);
+    EVP_CIPHER_CTX_ctrl(c->ctx, EVP_CTRL_GCM_SET_TAG, c->tag_len, &dummy_tag);
 
-    rv = EVP_Cipher(&c->ctx, NULL, aad, aad_len);
+    rv = EVP_Cipher(c->ctx, NULL, aad, aad_len);
     if (rv != aad_len) {
         return (srtp_err_status_algo_fail);
     } else {
@@ -282,16 +281,19 @@ static srtp_err_status_t srtp_aes_gcm_openssl_set_aad (srtp_aes_gcm_ctx_t *c, co
  *	buf	data to encrypt
  *	enc_len	length of encrypt buffer
  */
-static srtp_err_status_t srtp_aes_gcm_openssl_encrypt (srtp_aes_gcm_ctx_t *c, unsigned char *buf, unsigned int *enc_len)
+static srtp_err_status_t srtp_aes_gcm_openssl_encrypt(void *cv,
+                                                      unsigned char *buf,
+                                                      unsigned int *enc_len)
 {
-    if (c->dir != direction_encrypt && c->dir != direction_decrypt) {
+    srtp_aes_gcm_ctx_t *c = (srtp_aes_gcm_ctx_t *)cv;
+    if (c->dir != srtp_direction_encrypt && c->dir != srtp_direction_decrypt) {
         return (srtp_err_status_bad_param);
     }
 
     /*
      * Encrypt the data
      */
-    EVP_Cipher(&c->ctx, buf, buf, *enc_len);
+    EVP_Cipher(c->ctx, buf, buf, *enc_len);
 
     return (srtp_err_status_ok);
 }
@@ -307,17 +309,20 @@ static srtp_err_status_t srtp_aes_gcm_openssl_encrypt (srtp_aes_gcm_ctx_t *c, un
  *	buf	data to encrypt
  *	len	length of encrypt buffer
  */
-static srtp_err_status_t srtp_aes_gcm_openssl_get_tag (srtp_aes_gcm_ctx_t *c, uint8_t *buf, uint32_t *len)
+static srtp_err_status_t srtp_aes_gcm_openssl_get_tag(void *cv,
+                                                      uint8_t *buf,
+                                                      uint32_t *len)
 {
+    srtp_aes_gcm_ctx_t *c = (srtp_aes_gcm_ctx_t *)cv;
     /*
      * Calculate the tag
      */
-    EVP_Cipher(&c->ctx, NULL, NULL, 0);
+    EVP_Cipher(c->ctx, NULL, NULL, 0);
 
     /*
      * Retreive the tag
      */
-    EVP_CIPHER_CTX_ctrl(&c->ctx, EVP_CTRL_GCM_GET_TAG, c->tag_len, buf);
+    EVP_CIPHER_CTX_ctrl(c->ctx, EVP_CTRL_GCM_GET_TAG, c->tag_len, buf);
 
     /*
      * Increase encryption length by desired tag size
@@ -327,7 +332,6 @@ static srtp_err_status_t srtp_aes_gcm_openssl_get_tag (srtp_aes_gcm_ctx_t *c, ui
     return (srtp_err_status_ok);
 }
 
-
 /*
  * This function decrypts a buffer using AES GCM mode
  *
@@ -336,23 +340,26 @@ static srtp_err_status_t srtp_aes_gcm_openssl_get_tag (srtp_aes_gcm_ctx_t *c, ui
  *	buf	data to encrypt
  *	enc_len	length of encrypt buffer
  */
-static srtp_err_status_t srtp_aes_gcm_openssl_decrypt (srtp_aes_gcm_ctx_t *c, unsigned char *buf, unsigned int *enc_len)
+static srtp_err_status_t srtp_aes_gcm_openssl_decrypt(void *cv,
+                                                      unsigned char *buf,
+                                                      unsigned int *enc_len)
 {
-    if (c->dir != direction_encrypt && c->dir != direction_decrypt) {
+    srtp_aes_gcm_ctx_t *c = (srtp_aes_gcm_ctx_t *)cv;
+    if (c->dir != srtp_direction_encrypt && c->dir != srtp_direction_decrypt) {
         return (srtp_err_status_bad_param);
     }
 
     /*
      * Set the tag before decrypting
      */
-    EVP_CIPHER_CTX_ctrl(&c->ctx, EVP_CTRL_GCM_SET_TAG, c->tag_len,
+    EVP_CIPHER_CTX_ctrl(c->ctx, EVP_CTRL_GCM_SET_TAG, c->tag_len,
                         buf + (*enc_len - c->tag_len));
-    EVP_Cipher(&c->ctx, buf, buf, *enc_len - c->tag_len);
+    EVP_Cipher(c->ctx, buf, buf, *enc_len - c->tag_len);
 
     /*
      * Check the tag
      */
-    if (EVP_Cipher(&c->ctx, NULL, NULL, 0)) {
+    if (EVP_Cipher(c->ctx, NULL, NULL, 0)) {
         return (srtp_err_status_auth_fail);
     }
 
@@ -365,32 +372,36 @@ static srtp_err_status_t srtp_aes_gcm_openssl_decrypt (srtp_aes_gcm_ctx_t *c, un
     return (srtp_err_status_ok);
 }
 
-
-
 /*
  * Name of this crypto engine
  */
-static const char srtp_aes_gcm_128_openssl_description[] = "AES-128 GCM using openssl";
-static const char srtp_aes_gcm_256_openssl_description[] = "AES-256 GCM using openssl";
-
+static const char srtp_aes_gcm_128_openssl_description[] =
+    "AES-128 GCM using openssl";
+static const char srtp_aes_gcm_256_openssl_description[] =
+    "AES-256 GCM using openssl";
 
 /*
  * KAT values for AES self-test.  These
  * values we're derived from independent test code
  * using OpenSSL.
  */
-static const uint8_t srtp_aes_gcm_test_case_0_key[SRTP_AES_128_GCM_KEYSIZE_WSALT] = {
+/* clang-format off */
+static const uint8_t srtp_aes_gcm_test_case_0_key[SRTP_AES_GCM_128_KEY_LEN_WSALT] = {
     0xfe, 0xff, 0xe9, 0x92, 0x86, 0x65, 0x73, 0x1c,
     0x6d, 0x6a, 0x8f, 0x94, 0x67, 0x30, 0x83, 0x08,
     0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
     0x09, 0x0a, 0x0b, 0x0c,
 };
+/* clang-format on */
 
+/* clang-format off */
 static uint8_t srtp_aes_gcm_test_case_0_iv[12] = {
     0xca, 0xfe, 0xba, 0xbe, 0xfa, 0xce, 0xdb, 0xad,
     0xde, 0xca, 0xf8, 0x88
 };
+/* clang-format on */
 
+/* clang-format off */
 static const uint8_t srtp_aes_gcm_test_case_0_plaintext[60] =  {
     0xd9, 0x31, 0x32, 0x25, 0xf8, 0x84, 0x06, 0xe5,
     0xa5, 0x59, 0x09, 0xc5, 0xaf, 0xf5, 0x26, 0x9a,
@@ -402,12 +413,15 @@ static const uint8_t srtp_aes_gcm_test_case_0_plaintext[60] =  {
     0xba, 0x63, 0x7b, 0x39
 };
 
+/* clang-format off */
 static const uint8_t srtp_aes_gcm_test_case_0_aad[20] = {
     0xfe, 0xed, 0xfa, 0xce, 0xde, 0xad, 0xbe, 0xef,
     0xfe, 0xed, 0xfa, 0xce, 0xde, 0xad, 0xbe, 0xef,
     0xab, 0xad, 0xda, 0xd2
 };
+/* clang-format on */
 
+/* clang-format off */
 static const uint8_t srtp_aes_gcm_test_case_0_ciphertext[76] = {
     0x42, 0x83, 0x1e, 0xc2, 0x21, 0x77, 0x74, 0x24,
     0x4b, 0x72, 0x21, 0xb7, 0x84, 0xd0, 0xd4, 0x9c,
@@ -421,9 +435,10 @@ static const uint8_t srtp_aes_gcm_test_case_0_ciphertext[76] = {
     0x5b, 0xc9, 0x4f, 0xbc, 0x32, 0x21, 0xa5, 0xdb,
     0x94, 0xfa, 0xe9, 0x5a, 0xe7, 0x12, 0x1a, 0x47,
 };
+/* clang-format on */
 
 static const srtp_cipher_test_case_t srtp_aes_gcm_test_case_0a = {
-    SRTP_AES_128_GCM_KEYSIZE_WSALT,      /* octets in key            */
+    SRTP_AES_GCM_128_KEY_LEN_WSALT,      /* octets in key            */
     srtp_aes_gcm_test_case_0_key,        /* key                      */
     srtp_aes_gcm_test_case_0_iv,         /* packet index             */
     60,                                  /* octets in plaintext      */
@@ -432,12 +447,12 @@ static const srtp_cipher_test_case_t srtp_aes_gcm_test_case_0a = {
     srtp_aes_gcm_test_case_0_ciphertext, /* ciphertext  + tag        */
     20,                                  /* octets in AAD            */
     srtp_aes_gcm_test_case_0_aad,        /* AAD                      */
-    GCM_AUTH_TAG_LEN_8,
+    GCM_AUTH_TAG_LEN_8,                  /* */
     NULL                                 /* pointer to next testcase */
 };
 
 static const srtp_cipher_test_case_t srtp_aes_gcm_test_case_0 = {
-    SRTP_AES_128_GCM_KEYSIZE_WSALT,      /* octets in key            */
+    SRTP_AES_GCM_128_KEY_LEN_WSALT,      /* octets in key            */
     srtp_aes_gcm_test_case_0_key,        /* key                      */
     srtp_aes_gcm_test_case_0_iv,         /* packet index             */
     60,                                  /* octets in plaintext      */
@@ -446,25 +461,29 @@ static const srtp_cipher_test_case_t srtp_aes_gcm_test_case_0 = {
     srtp_aes_gcm_test_case_0_ciphertext, /* ciphertext  + tag        */
     20,                                  /* octets in AAD            */
     srtp_aes_gcm_test_case_0_aad,        /* AAD                      */
-    GCM_AUTH_TAG_LEN,
+    GCM_AUTH_TAG_LEN,                    /* */
     &srtp_aes_gcm_test_case_0a           /* pointer to next testcase */
 };
 
-static const uint8_t srtp_aes_gcm_test_case_1_key[SRTP_AES_256_GCM_KEYSIZE_WSALT] = {
+/* clang-format off */
+static const uint8_t srtp_aes_gcm_test_case_1_key[SRTP_AES_GCM_256_KEY_LEN_WSALT] = {
     0xfe, 0xff, 0xe9, 0x92, 0x86, 0x65, 0x73, 0x1c,
     0xa5, 0x59, 0x09, 0xc5, 0x54, 0x66, 0x93, 0x1c,
     0xaf, 0xf5, 0x26, 0x9a, 0x21, 0xd5, 0x14, 0xb2,
     0x6d, 0x6a, 0x8f, 0x94, 0x67, 0x30, 0x83, 0x08,
     0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
     0x09, 0x0a, 0x0b, 0x0c,
-
 };
+/* clang-format on */
 
+/* clang-format off */
 static uint8_t srtp_aes_gcm_test_case_1_iv[12] = {
     0xca, 0xfe, 0xba, 0xbe, 0xfa, 0xce, 0xdb, 0xad,
     0xde, 0xca, 0xf8, 0x88
 };
+/* clang-format on */
 
+/* clang-format off */
 static const uint8_t srtp_aes_gcm_test_case_1_plaintext[60] =  {
     0xd9, 0x31, 0x32, 0x25, 0xf8, 0x84, 0x06, 0xe5,
     0xa5, 0x59, 0x09, 0xc5, 0xaf, 0xf5, 0x26, 0x9a,
@@ -475,13 +494,17 @@ static const uint8_t srtp_aes_gcm_test_case_1_plaintext[60] =  {
     0xb1, 0x6a, 0xed, 0xf5, 0xaa, 0x0d, 0xe6, 0x57,
     0xba, 0x63, 0x7b, 0x39
 };
+/* clang-format on */
 
+/* clang-format off */
 static const uint8_t srtp_aes_gcm_test_case_1_aad[20] = {
     0xfe, 0xed, 0xfa, 0xce, 0xde, 0xad, 0xbe, 0xef,
     0xfe, 0xed, 0xfa, 0xce, 0xde, 0xad, 0xbe, 0xef,
     0xab, 0xad, 0xda, 0xd2
 };
+/* clang-format on */
 
+/* clang-format off */
 static const uint8_t srtp_aes_gcm_test_case_1_ciphertext[76] = {
     0x0b, 0x11, 0xcf, 0xaf, 0x68, 0x4d, 0xae, 0x46,
     0xc7, 0x90, 0xb8, 0x8e, 0xb7, 0x6a, 0x76, 0x2a,
@@ -495,9 +518,10 @@ static const uint8_t srtp_aes_gcm_test_case_1_ciphertext[76] = {
     0x45, 0xbc, 0x03, 0xe6, 0xe1, 0xac, 0x0a, 0x9f,
     0x81, 0xcb, 0x8e, 0x5b, 0x46, 0x65, 0x63, 0x1d,
 };
+/* clang-format on */
 
 static const srtp_cipher_test_case_t srtp_aes_gcm_test_case_1a = {
-    SRTP_AES_256_GCM_KEYSIZE_WSALT,      /* octets in key            */
+    SRTP_AES_GCM_256_KEY_LEN_WSALT,      /* octets in key            */
     srtp_aes_gcm_test_case_1_key,        /* key                      */
     srtp_aes_gcm_test_case_1_iv,         /* packet index             */
     60,                                  /* octets in plaintext      */
@@ -506,12 +530,12 @@ static const srtp_cipher_test_case_t srtp_aes_gcm_test_case_1a = {
     srtp_aes_gcm_test_case_1_ciphertext, /* ciphertext  + tag        */
     20,                                  /* octets in AAD            */
     srtp_aes_gcm_test_case_1_aad,        /* AAD                      */
-    GCM_AUTH_TAG_LEN_8,
+    GCM_AUTH_TAG_LEN_8,                  /* */
     NULL                                 /* pointer to next testcase */
 };
 
 static const srtp_cipher_test_case_t srtp_aes_gcm_test_case_1 = {
-    SRTP_AES_256_GCM_KEYSIZE_WSALT,      /* octets in key            */
+    SRTP_AES_GCM_256_KEY_LEN_WSALT,      /* octets in key            */
     srtp_aes_gcm_test_case_1_key,        /* key                      */
     srtp_aes_gcm_test_case_1_iv,         /* packet index             */
     60,                                  /* octets in plaintext      */
@@ -520,43 +544,40 @@ static const srtp_cipher_test_case_t srtp_aes_gcm_test_case_1 = {
     srtp_aes_gcm_test_case_1_ciphertext, /* ciphertext  + tag        */
     20,                                  /* octets in AAD            */
     srtp_aes_gcm_test_case_1_aad,        /* AAD                      */
-    GCM_AUTH_TAG_LEN,
+    GCM_AUTH_TAG_LEN,                    /* */
     &srtp_aes_gcm_test_case_1a           /* pointer to next testcase */
 };
 
 /*
  * This is the vector function table for this crypto engine.
  */
-const srtp_cipher_type_t srtp_aes_gcm_128_openssl = {
-    (cipher_alloc_func_t)srtp_aes_gcm_openssl_alloc,
-    (cipher_dealloc_func_t)srtp_aes_gcm_openssl_dealloc,
-    (cipher_init_func_t)srtp_aes_gcm_openssl_context_init,
-    (cipher_set_aad_func_t)srtp_aes_gcm_openssl_set_aad,
-    (cipher_encrypt_func_t)srtp_aes_gcm_openssl_encrypt,
-    (cipher_decrypt_func_t)srtp_aes_gcm_openssl_decrypt,
-    (cipher_set_iv_func_t)srtp_aes_gcm_openssl_set_iv,
-    (cipher_get_tag_func_t)srtp_aes_gcm_openssl_get_tag,
-    (const char*)srtp_aes_gcm_128_openssl_description,
-    (const srtp_cipher_test_case_t*)&srtp_aes_gcm_test_case_0,
-    (srtp_debug_module_t*)&srtp_mod_aes_gcm,
-    (srtp_cipher_type_id_t)SRTP_AES_128_GCM
+const srtp_cipher_type_t srtp_aes_gcm_128 = {
+    srtp_aes_gcm_openssl_alloc,
+    srtp_aes_gcm_openssl_dealloc,
+    srtp_aes_gcm_openssl_context_init,
+    srtp_aes_gcm_openssl_set_aad,
+    srtp_aes_gcm_openssl_encrypt,
+    srtp_aes_gcm_openssl_decrypt,
+    srtp_aes_gcm_openssl_set_iv,
+    srtp_aes_gcm_openssl_get_tag,
+    srtp_aes_gcm_128_openssl_description,
+    &srtp_aes_gcm_test_case_0,
+    SRTP_AES_GCM_128
 };
 
 /*
  * This is the vector function table for this crypto engine.
  */
-const srtp_cipher_type_t srtp_aes_gcm_256_openssl = {
-    (cipher_alloc_func_t)srtp_aes_gcm_openssl_alloc,
-    (cipher_dealloc_func_t)srtp_aes_gcm_openssl_dealloc,
-    (cipher_init_func_t)srtp_aes_gcm_openssl_context_init,
-    (cipher_set_aad_func_t)srtp_aes_gcm_openssl_set_aad,
-    (cipher_encrypt_func_t)srtp_aes_gcm_openssl_encrypt,
-    (cipher_decrypt_func_t)srtp_aes_gcm_openssl_decrypt,
-    (cipher_set_iv_func_t)srtp_aes_gcm_openssl_set_iv,
-    (cipher_get_tag_func_t)srtp_aes_gcm_openssl_get_tag,
-    (const char*)srtp_aes_gcm_256_openssl_description,
-    (const srtp_cipher_test_case_t*)&srtp_aes_gcm_test_case_1,
-    (srtp_debug_module_t*)&srtp_mod_aes_gcm,
-    (srtp_cipher_type_id_t)SRTP_AES_256_GCM
+const srtp_cipher_type_t srtp_aes_gcm_256 = {
+    srtp_aes_gcm_openssl_alloc,
+    srtp_aes_gcm_openssl_dealloc,
+    srtp_aes_gcm_openssl_context_init,
+    srtp_aes_gcm_openssl_set_aad,
+    srtp_aes_gcm_openssl_encrypt,
+    srtp_aes_gcm_openssl_decrypt,
+    srtp_aes_gcm_openssl_set_iv,
+    srtp_aes_gcm_openssl_get_tag,
+    srtp_aes_gcm_256_openssl_description,
+    &srtp_aes_gcm_test_case_1,
+    SRTP_AES_GCM_256
 };
-
