@@ -42,45 +42,70 @@
  *
  */
 
+#include "config.h"
 #include "util.h"
 
 #include <string.h>
 #include <stdint.h>
 
-char bit_string[MAX_PRINT_STRING_LEN];
+/* include space for null terminator */
+char bit_string[MAX_PRINT_STRING_LEN + 1];
 
-static inline int hex_char_to_nibble (uint8_t c)
+static inline int hex_char_to_nibble(uint8_t c)
 {
     switch (c) {
-    case ('0'): return 0x0;
-    case ('1'): return 0x1;
-    case ('2'): return 0x2;
-    case ('3'): return 0x3;
-    case ('4'): return 0x4;
-    case ('5'): return 0x5;
-    case ('6'): return 0x6;
-    case ('7'): return 0x7;
-    case ('8'): return 0x8;
-    case ('9'): return 0x9;
-    case ('a'): return 0xa;
-    case ('A'): return 0xa;
-    case ('b'): return 0xb;
-    case ('B'): return 0xb;
-    case ('c'): return 0xc;
-    case ('C'): return 0xc;
-    case ('d'): return 0xd;
-    case ('D'): return 0xd;
-    case ('e'): return 0xe;
-    case ('E'): return 0xe;
-    case ('f'): return 0xf;
-    case ('F'): return 0xf;
-    default: return -1; /* this flags an error */
+    case ('0'):
+        return 0x0;
+    case ('1'):
+        return 0x1;
+    case ('2'):
+        return 0x2;
+    case ('3'):
+        return 0x3;
+    case ('4'):
+        return 0x4;
+    case ('5'):
+        return 0x5;
+    case ('6'):
+        return 0x6;
+    case ('7'):
+        return 0x7;
+    case ('8'):
+        return 0x8;
+    case ('9'):
+        return 0x9;
+    case ('a'):
+        return 0xa;
+    case ('A'):
+        return 0xa;
+    case ('b'):
+        return 0xb;
+    case ('B'):
+        return 0xb;
+    case ('c'):
+        return 0xc;
+    case ('C'):
+        return 0xc;
+    case ('d'):
+        return 0xd;
+    case ('D'):
+        return 0xd;
+    case ('e'):
+        return 0xe;
+    case ('E'):
+        return 0xe;
+    case ('f'):
+        return 0xf;
+    case ('F'):
+        return 0xf;
+    default:
+        return -1; /* this flags an error */
     }
     /* NOTREACHED */
     return -1; /* this keeps compilers from complaining */
 }
 
-uint8_t nibble_to_hex_char (uint8_t nibble)
+uint8_t nibble_to_hex_char(uint8_t nibble)
 {
     char buf[16] = { '0', '1', '2', '3', '4', '5', '6', '7',
                      '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
@@ -92,7 +117,7 @@ uint8_t nibble_to_hex_char (uint8_t nibble)
  * hex_string_to_octet_string converts a hexadecimal string
  * of length 2 * len to a raw octet string of length len
  */
-int hex_string_to_octet_string (char *raw, char *hex, int len)
+int hex_string_to_octet_string(char *raw, char *hex, int len)
 {
     uint8_t x;
     int tmp;
@@ -118,9 +143,9 @@ int hex_string_to_octet_string (char *raw, char *hex, int len)
     return hex_len;
 }
 
-char * octet_string_hex_string (const void *s, int length)
+char *octet_string_hex_string(const void *s, int length)
 {
-    const uint8_t *str = (const uint8_t*)s;
+    const uint8_t *str = (const uint8_t *)s;
     int i;
 
     /* double length, since one octet takes two hex characters */
@@ -128,11 +153,11 @@ char * octet_string_hex_string (const void *s, int length)
 
     /* truncate string if it would be too long */
     if (length > MAX_PRINT_STRING_LEN) {
-        length = MAX_PRINT_STRING_LEN - 1;
+        length = MAX_PRINT_STRING_LEN;
     }
 
     for (i = 0; i < length; i += 2) {
-        bit_string[i]   = nibble_to_hex_char(*str >> 4);
+        bit_string[i] = nibble_to_hex_char(*str >> 4);
         bit_string[i + 1] = nibble_to_hex_char(*str++ & 0xF);
     }
     bit_string[i] = 0; /* null terminate string */
@@ -142,9 +167,9 @@ char * octet_string_hex_string (const void *s, int length)
 static const char b64chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                                "abcdefghijklmnopqrstuvwxyz0123456789+/";
 
-static int base64_block_to_octet_triple (char *out, char *in)
+static int base64_block_to_octet_triple(char *out, char *in)
 {
-    unsigned char sextets[4] = {0};
+    unsigned char sextets[4] = { 0 };
     int j = 0;
     int i;
 
@@ -152,7 +177,9 @@ static int base64_block_to_octet_triple (char *out, char *in)
         char *p = strchr(b64chars, in[i]);
         if (p != NULL) {
             sextets[i] = p - b64chars;
-        } else{  j++; }
+        } else {
+            j++;
+        }
     }
 
     out[0] = (sextets[0] << 2) | (sextets[1] >> 4);
@@ -165,7 +192,7 @@ static int base64_block_to_octet_triple (char *out, char *in)
     return j;
 }
 
-int base64_string_to_octet_string (char *out, int *pad, char *in, int len)
+int base64_string_to_octet_string(char *out, int *pad, char *in, int len)
 {
     int k = 0;
     int i = 0;
